"""
Apps.
"""
# flake8: noqa
# pylint: disable-all
from django.apps import AppConfig


class DjangoTelegramLoginConfig(AppConfig):
    """
    DjangoTelegramLoginConfig.
    """
    name = 'django_telegram_login'



"""
Tests.
"""
# flake8: noqa
# pylint: disable-all
import time
import unittest

import mock

from django_telegram_login.authentication import verify_telegram_authentication
from django_telegram_login.widgets.constants import (
    SMALL,
    MEDIUM,
    LARGE,
    DISABLE_USER_PHOTO,
)
from django_telegram_login.widgets.generator import (
    create_callback_login_widget,
    create_redirect_login_widget,
)
from django_telegram_login.errors import (
    NotTelegramDataError,
    TelegramDataIsOutdatedError,
)


class TestAuthentication(unittest.TestCase):

    def setUp(self):
        self.bot_token = '459236585:AAEee0Ba4fRijf1BRNbBO9W-Ar15F2xgV98'

        self.request_data = {
            'id': '299661134',
            'first_name': 'Dmytro',
            'last_name': 'Striletskyi',
            'username': 'dmytrostriletskyi',
            'photo_url': 'https://t.me/i/userpic/320/dmytrostriletskyi.jpg',
            'auth_date': '1518392724',
            'hash': '92ee8156a1482919843bfbaed2a91839f6594b2b98d884046c48ff58fa3a5ace'
        }

    @mock.patch('django_telegram_login.authentication.time.time')
    def test_ok_data(self, mock_time):
        """
        Received data is correct.
        """
        # auth_date and time now in unix datetime format are the same
        mock_time.return_value = 1518392724

        expected = self.request_data.copy()
        expected.pop('hash', None)

        result = verify_telegram_authentication(self.bot_token, self.request_data)
        self.assertEqual(expected, result)

    @mock.patch('django_telegram_login.authentication.time.time')
    def test_wrong_token(self, mock_time):
        """
        Wrong token.
        """
        # auth_date and time now in unix datetime format are the same
        mock_time.return_value = 1518392724

        request_data = self.request_data.copy()
        request_data['hash'] = '92ee8156a1482919843bfbaed2a91839f6594b2b98d884046c48ff58fa3a13c29'

        with self.assertRaises(NotTelegramDataError):
            verify_telegram_authentication(self.bot_token, request_data)

    def test_outdatet_data(self):
        """
        Outdated data.
        """
        request_data = self.request_data.copy()
        request_data['auth_date'] = '10'

        with self.assertRaises(TelegramDataIsOutdatedError):
            verify_telegram_authentication(self.bot_token, request_data)


class TestWidgetGenerator(unittest.TestCase):

    def setUp(self):

        self.bot_name = 'test_bot'
        self.redirect_url = 'https://django-telegram-login.com'

        self.expected_callback_small = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="small" data-onauth="onTelegramAuth(user)" data-request-access="write"></script>"""
        self.expected_callback_medium = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="medium" data-onauth="onTelegramAuth(user)" data-request-access="write"></script>"""
        self.expected_callback_large = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="large" data-onauth="onTelegramAuth(user)" data-request-access="write"></script>"""
        self.expected_callback_no_photo = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="small" data-userpic="false" data-onauth="onTelegramAuth(user)" data-request-access="write"></script>"""
        self.expected_callback_corner_radius = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="small" data-userpic="false" data-radius="1" data-onauth="onTelegramAuth(user)" data-request-access="write"></script>"""

        self.expected_redirect_small = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="small" data-auth-url="https://django-telegram-login.com" data-request-access="write"></script>"""
        self.expected_redirect_medium = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="medium" data-auth-url="https://django-telegram-login.com" data-request-access="write"></script>"""
        self.expected_redirect_large = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="large" data-auth-url="https://django-telegram-login.com" data-request-access="write"></script>"""
        self.expected_redirect_no_photo = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="small" data-userpic="false" data-auth-url="https://django-telegram-login.com" data-request-access="write"></script>"""
        self.expected_redirect_corner_radius = """<script async src="https://telegram.org/js/telegram-widget.js?3" data-telegram-login="test_bot" data-size="small" data-userpic="false" data-radius="10" data-auth-url="https://django-telegram-login.com" data-request-access="write"></script>"""

    def test_create_callback_small(self):
        result = create_callback_login_widget(self.bot_name, size=SMALL)
        self.assertEqual(self.expected_callback_small, result)

    def test_create_callback_medium(self):
        result = create_callback_login_widget(self.bot_name, size=MEDIUM)
        self.assertEqual(self.expected_callback_medium, result)

    def test_create_callback_large(self):
        result = create_callback_login_widget(self.bot_name, size=LARGE)
        self.assertEqual(self.expected_callback_large, result)

    def test_create_callback_corner_radius(self):
        result = create_callback_login_widget(self.bot_name, corner_radius=1, user_photo=DISABLE_USER_PHOTO)
        self.assertEqual(self.expected_callback_corner_radius, result)

    def test_create_callback_no_photo(self):
        result = create_callback_login_widget(self.bot_name, user_photo=DISABLE_USER_PHOTO)
        self.assertEqual(self.expected_callback_no_photo, result)

    def test_create_redirect_small(self):
        result = create_redirect_login_widget(self.redirect_url, self.bot_name, size=SMALL)
        self.assertEqual(self.expected_redirect_small, result)

    def test_create_redirect_medium(self):
        result = create_redirect_login_widget(self.redirect_url, self.bot_name, size=MEDIUM)
        self.assertEqual(self.expected_redirect_medium, result)

    def test_create_redirect_large(self):
        result = create_redirect_login_widget(self.redirect_url, self.bot_name, size=LARGE)
        self.assertEqual(self.expected_redirect_large, result)

    def test_create_redirect_corner_radius(self):
        result = create_redirect_login_widget(self.redirect_url, self.bot_name, corner_radius=10, user_photo=DISABLE_USER_PHOTO)
        self.assertEqual(self.expected_redirect_corner_radius, result)

    def test_create_redirect_no_photo(self):
        result = create_redirect_login_widget(self.redirect_url, self.bot_name, user_photo=DISABLE_USER_PHOTO)
        self.assertEqual(self.expected_redirect_no_photo, result)


if __name__ == '__main__':
    unittest.main()



"""
Widgets generator interface.
"""
from django_telegram_login.widgets.constants import SMALL


def create_callback_login_widget(
        bot_name, size=SMALL, corner_radius=None, user_photo=True, access_write=True
):
    """
    Create callback widget, that allows to handle user data in JavaSccript.
    """
    script_initital = '<script async src="https://telegram.org/js/telegram-widget.js?3" '
    bot = 'data-telegram-login="{}" '.format(bot_name)
    size = 'data-size="{}" '.format(size)
    userpic = 'data-userpic="{}" '.format(str(user_photo).lower()) if not user_photo else ''
    corner_radius = 'data-radius="{}" '.format(corner_radius) if corner_radius else ''
    onauth = 'data-onauth="onTelegramAuth(user)" '
    access = 'data-request-access="write"' if access_write else ''
    script_end = '></script>'

    widget_script = script_initital + bot + size + userpic + corner_radius + onauth + access + script_end
    return widget_script


def create_redirect_login_widget(
        redirect_url, bot_name, size=SMALL, corner_radius=None, user_photo=True, access_write=True
):
    """
    Create redirect widget, that allows to handle user data as get request params.
    """
    script_initital = '<script async src="https://telegram.org/js/telegram-widget.js?3" '
    bot = 'data-telegram-login="{}" '.format(bot_name)
    size = 'data-size="{}" '.format(size)
    userpic = 'data-userpic="{}" '.format(str(user_photo).lower()) if not user_photo else ''
    corner_radius = 'data-radius="{}" '.format(corner_radius) if corner_radius else ''
    redirect = 'data-auth-url="{}" '.format(redirect_url)
    access = 'data-request-access="write"' if access_write else ''
    script_end = '></script>'

    widget_script = script_initital + bot + size + userpic + corner_radius + redirect + access + script_end
    return widget_script



"""
Errors.
"""


class NotTelegramDataError(Exception):
    """
    Error for data that was hacked.
    """
    pass


class TelegramDataIsOutdatedError(Exception):
    """
    Error for outdated data.
    """
    pass



"""
Telegram login authentication.
"""
import hashlib
import hmac
import time

from django_telegram_login.errors import (
    NotTelegramDataError,
    TelegramDataIsOutdatedError,
)

ONE_DAY_IN_SECONDS = 86400


def verify_telegram_authentication(bot_token, request_data):
    """
    Check if received data from Telegram based on SHA and HMAC algothims.

    Instructions - https://core.telegram.org/widgets/login#checking-authorization
    """
    request_data = request_data.copy()

    received_hash = request_data['hash']
    auth_date = request_data['auth_date']

    request_data.pop('hash', None)
    request_data_alphabetical_order = sorted(request_data.items(), key=lambda x: x[0])

    data_check_string = []

    for data_pair in request_data_alphabetical_order:
        key, value = data_pair[0], data_pair[1]
        data_check_string.append(key + '=' + value)

    data_check_string = '\n'.join(data_check_string)

    secret_key = hashlib.sha256(bot_token.encode()).digest()
    _hash = hmac.new(secret_key, msg=data_check_string.encode(), digestmod=hashlib.sha256).hexdigest()

    unix_time_now = int(time.time())
    unix_time_auth_date = int(auth_date)

    if unix_time_now - unix_time_auth_date > ONE_DAY_IN_SECONDS:
        raise TelegramDataIsOutdatedError(
            'Authentication data is outdated. Authentication was received more than day ago.'
        )

    if _hash != received_hash:
        raise NotTelegramDataError(
            'This is not a Telegram data. Hash from recieved authentication data does not match'
            'with calculated hash based on bot token.'
        )

    return request_data



import os
from setuptools import find_packages, setup

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-telegram-login',
    version='0.2.0',
    packages = find_packages(
        exclude=[
            'login_app*',
            'django_telegram_tools',
        ]
    ),
    include_package_data=True,
    license='MIT',
    description='Telegram login interface and widgets for Django projects, that allows to handle user based on Telegram data.',
    url='https://github.com/dmytrostriletskyi/django-telegram-login',
    author='Dmytro Striletskyi',
    author_email='dmytro.striletskyi@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
